/*GPL*START*
 * 
 * Copyright (C) 1998 by Johannes Overmann <overmann@iname.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * *GPL*END*/  

#include "machine.h"
#include <ctype.h>
#include <stdlib.h>

void isort(void *base, size_t nmemb, size_t size,
			  int (*compar)(const void *, const void *)) {
  int k, l, m, stop;
  char *buffer;
  
  if ((buffer = (char*)malloc(size)) == NULL) {
    printf("Cannot allocate memory for insertion sort.\n");
    exit(-1);
  }
  for (k = 1; k < (int)nmemb; k++) {
    l = 0;
    stop = 0;
    while ((l < k) && (!stop)) {
      if (compar(((char *)base)+size*k, ((char *)base)+size*l) < 0) {
		  stop = 1;
      } else {
		  l++;
      }
    }
    if (stop) {
      memcpy(buffer, ((char *)base)+size*k, size);
      for (m = k-1; m >= l; m--) {
        memcpy(((char *)base)+size*(m+1), ((char *)base)+size*m, size);
      }
      /*memmove(((char *)base)+size*l, ((char *)base)+size*(l+1), size*(k-l));*/
      memcpy(((char *)base)+size*l, buffer, size);
    }
  }
  free(buffer);
}


TString str2machine(const char *p) {
   for(; (!isalpha(*p)) && *p; p++);
   int i;
   for(i=0; isalpha(p[i]); i++);
   TString s(p,i);
   s.capitalize();
   return s;
}

int machcmp_load1(const void *pp1, const void *pp2) {
   const Machine *p1 = (const Machine*)pp1;
   const Machine *p2 = (const Machine*)pp2;
   if(p1->load1 < p2->load1) return -1;
   if(p1->load1 > p2->load1) return +1;
   return 0;
}

int machcmp_load1_r(const void *pp1, const void *pp2) {
   const Machine *p1 = (const Machine*)pp1;
   const Machine *p2 = (const Machine*)pp2;
   if(p1->load1 > p2->load1) return -1;
   if(p1->load1 < p2->load1) return +1;
   return 0;
}

int machcmp_load2(const void *pp1, const void *pp2) {
   const Machine *p1 = (const Machine*)pp1;
   const Machine *p2 = (const Machine*)pp2;
   if(p1->load2 < p2->load2) return -1;
   if(p1->load2 > p2->load2) return +1;
   return 0;
}

int machcmp_load2_r(const void *pp1, const void *pp2) {
   const Machine *p1 = (const Machine*)pp1;
   const Machine *p2 = (const Machine*)pp2;
   if(p1->load2 > p2->load2) return -1;
   if(p1->load2 < p2->load2) return +1;
   return 0;
}

int machcmp_load3(const void *pp1, const void *pp2) {
   const Machine *p1 = (const Machine*)pp1;
   const Machine *p2 = (const Machine*)pp2;
   if(p1->load3 < p2->load3) return -1;
   if(p1->load3 > p2->load3) return +1;
   return 0;
}

int machcmp_load3_r(const void *pp1, const void *pp2) {
   const Machine *p1 = (const Machine*)pp1;
   const Machine *p2 = (const Machine*)pp2;
   if(p1->load3 > p2->load3) return -1;
   if(p1->load3 < p2->load3) return +1;
   return 0;
}


int machcmp_name(const void *pp1, const void *pp2) {
   const Machine *p1 = (const Machine*)pp1;
   const Machine *p2 = (const Machine*)pp2;
   if(p1->name < p2->name) return -1;
   if(p1->name > p2->name) return +1;
   return 0;
}

int machcmp_name_r(const void *pp1, const void *pp2) {
   const Machine *p1 = (const Machine*)pp1;
   const Machine *p2 = (const Machine*)pp2;
   if(p1->name > p2->name) return -1;
   if(p1->name < p2->name) return +1;
   return 0;
}


int machcmp_ram(const void *pp1, const void *pp2) {
   const Machine *p1 = (const Machine*)pp1;
   const Machine *p2 = (const Machine*)pp2;
   if(p1->ram < p2->ram) return -1;
   if(p1->ram > p2->ram) return +1;
   return 0;
}

int machcmp_ram_r(const void *pp1, const void *pp2) {
   const Machine *p1 = (const Machine*)pp1;
   const Machine *p2 = (const Machine*)pp2;
   if(p1->ram > p2->ram) return -1;
   if(p1->ram < p2->ram) return +1;
   return 0;
}


int machcmp_group(const void *pp1, const void *pp2) {
   const Machine *p1 = (const Machine*)pp1;
   const Machine *p2 = (const Machine*)pp2;
   if(p1->group < p2->group) return -1;
   if(p1->group > p2->group) return +1;
   return 0;
}


int machcmp_group_r(const void *pp1, const void *pp2) {
   const Machine *p1 = (const Machine*)pp1;
   const Machine *p2 = (const Machine*)pp2;
   if(p1->group > p2->group) return -1;
   if(p1->group < p2->group) return +1;
   return 0;
}


int machcmp_arch(const void *pp1, const void *pp2) {
   const Machine *p1 = (const Machine*)pp1;
   const Machine *p2 = (const Machine*)pp2;
   if(p1->arch < p2->arch) return -1;
   if(p1->arch > p2->arch) return +1;
   return 0;
}


int machcmp_arch_r(const void *pp1, const void *pp2) {
   const Machine *p1 = (const Machine*)pp1;
   const Machine *p2 = (const Machine*)pp2;
   if(p1->arch > p2->arch) return -1;
   if(p1->arch < p2->arch) return +1;
   return 0;
}




